searchData={"items":[{"type":"module","title":"Valspec","doc":"# Valspec\n\n**An elixir library for Phoenix that generates Swagger documentation and performs parameter validation.**","ref":"Valspec.html"},{"type":"module","title":"Installation - Valspec","doc":"The latest version of Valspec is built on top [open_api_spex](https://github.com/open-api-spex/open_api_spex). You will need to install it along with the Valspec dependency:\n\n```elixir\ndef deps do\n  [\n    {:open_api_spex, \"~> 3.21\"},\n    {:valspec, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"Valspec.html#module-installation"},{"type":"module","title":"Adding to your project - Valspec","doc":"","ref":"Valspec.html#module-adding-to-your-project"},{"type":"module","title":"Define your spec - Valspec","doc":"The first thing you will need to do is define your `open_api_spex` spec definition.\nVisit the [Generate Spec](https://github.com/open-api-spex/open_api_spex?tab=readme-ov-file#generate-spec) section in the github repo for more information.","ref":"Valspec.html#module-define-your-spec"},{"type":"module","title":"Serving your spec (Running Swagger UI) - Valspec","doc":"Visit the [Serve Spec](https://github.com/open-api-spex/open_api_spex?tab=readme-ov-file#serve-the-spec) section in the github repo for more information on how to run the UI.","ref":"Valspec.html#module-serving-your-spec-running-swagger-ui"},{"type":"module","title":"Add Valspec to your controller - Valspec","doc":"```elixir\n  defmodule MyAppWeb.ExampleUsersController do\n    use MyAppWeb, :controller\n    use Valspec.Controller\n\n    valspec_create :create_user, summary: \"Creates a User\" do\n      required(:first_name, :string, example: \"Greg\")\n      required(:role, :enum, values: [:admin, :normal], default: :normal)\n      optional(:last_name, :string, example: \"Jones\")\n      optional(:age, :integer, minimum: 18)\n    end\n\n    def create(conn, params) do\n      # `valspec_validate/2` - validates a map of parameters. Returns:\n      #    - {:ok, map()} when params are valid\n      #    - {:error, Ecto.%Changeset{}} when params are invalid\n      with {:ok, user_params} <- valspec_validate(:create_user, params) do\n        ...\n      else\n        {:error, %Ecto.Changeset{} = _changeset} ->\n          ...\n      end\n    end\n  end\n```","ref":"Valspec.html#module-add-valspec-to-your-controller"},{"type":"module","title":"Creating a schema - Valspec","doc":"Using `valspex_schema/1` allows you to define response or callback schemas for your documentation. `valspex_schema/1` uses Ecto `embedded_schema` under the hood and uses its\nsyntax:\n\n```elixir\n  defmodule MyAppWeb.Users.Schema do\n    use Valspec.Schema\n\n    valspec_schema do\n      field :id, :uuid, example: \"82b557d1-0000-0000-0000-55df16add1b5\"\n      field :first_name, :string, example: \"Greg\"\n      field :last_name, :string, example: \"Jones\"\n      field :role, :enum, values: [:admin, :normal]\n      field :age, :integer\n    end\n  end\n\n  # You can use a valspec schema within other valspec schemas via `embeds_one/2` \n  # or `embeds_many/2`\n  defmodule MyApp.Users.SucessResponse do\n    use Valspec.Schema\n\n    valspec_schema do\n      embeds_one :data, MyAppWeb.Users.Schema\n    end\n  end\n```\n\nYou can use the schema by using the `default_response_schema` opt on init or by passing the `response_schema` to the valspec operation:\n\n```elixir\n  defmodule MyAppWeb.ExampleUsersController do\n    use MyAppWeb, :controller\n    use Valspec.Controller,\n    # Applies to all actions unless overridden\n    default_response_schema: MyApp.Users.SucessResponse\n    \n    # Uses MyApp.Users.SucessResponse\n    valspec_create :create_user, summary: \"Creates a User\" do\n      required(:first_name, :string, example: \"Greg\", nullable: true)\n    end\n\n    valspec_update \n      :update_user, \n      summary: \"Updates a User\"\n      # Overrides default MyApp.Users.SucessResponse\n      response_schema: MyApp.Users.UpdateSucessResponse do\n      required(:first_name, :string, example: \"Greg\")\n    end\n  end\n```","ref":"Valspec.html#module-creating-a-schema"},{"type":"module","title":"Using schemas for type safety - Valspec","doc":"You can use your schemas directly in your responses to ensure your documentation is always up-to-date like so:\n\n```elixir\ndefmodule MyAppWeb.ExampleUsers.JSON do \n  def create(%{user: user}) do\n    %MyApp.Users.SucessResponse{data: data(user)}\n  end\n\n  defp data(%User{} = user) do\n    %MyAppWeb.Users.Schema{\n      id: user.id,\n      first_name: user.first_name,\n      last_name: user.last_name,\n      role: user.role,\n      age: user.age\n    }\n  end\nend\n```","ref":"Valspec.html#module-using-schemas-for-type-safety"},{"type":"module","title":"List of operations - Valspec","doc":"The following macros are exposed:\n\n  * `valspec_create/3`- define specs for a controller's #create/2 action\n  * `valspec_update/3`- define specs for a controller's #update/2 action\n  * `valspec_index/1` - define specs for a controller's #index/2 action\n  * `valspec_delete/1` - define specs for a controller's #delete/2 action\n  * `valspec_show/1` - define specs for a controller's #show/2 action\n  * `valspec_custom/2` - define specs for a custom controller action.\n  * `valspec_custom/4` - define specs for a custom controller action.\n  * `valspec_schema/1`- defines a struct that can used in defining responses and callbacks.\n  * `valspec_validate/2` - validates a map of parameters.\n    Returns:\n      - `{:ok, map()}` when params are valid\n      - `{:error, Ecto.%Changeset{}}` when params are invalid","ref":"Valspec.html#module-list-of-operations"},{"type":"module","title":"Examples - Valspec","doc":"There is an example phoenix project in this repo under `/examples/my_app`. You can run the app and navigate to the `/swaggerui#` to see the example","ref":"Valspec.html#module-examples"},{"type":"module","title":"Valspec.Controller","doc":"A module to conveniently generate valspec definitions within Phoenix controllers.","ref":"Valspec.Controller.html"},{"type":"module","title":"Example - Valspec.Controller","doc":"defmodule MyAppWeb.ExampleUsersController do\n      use MyAppWeb, :controller\n      use Valspec.Controller\n\n      valspec_create :create_user, summary: \"Creates a User\" do\n        required(:first_name, :string, example: \"Greg\")\n        required(:role, :enum, values: [:admin, :normal], default: :normal)\n        optional(:last_name, :string, example: \"Jones\")\n        optional(:age, :integer, minimum: 18)\n      end\n\n      def create(conn, params) do\n            # `valspec_validate/2` - validates a map of parameters. Returns:\n            #    - {:ok, map()} when params are valid\n            #    - {:error, Ecto.%Changeset{}} when params are invalid\n        with {:ok, user_params} <- valspec_validate(:create_user, params) do\n          ...\n        else\n          {:error, %Ecto.Changeset{}} ->\n            ...\n        end\n      end\n    end\n\n\n# Options:\n  * `tags`- the Swagger tags for the module. Example: [tags: [\"V3 Clients Api\"]].\n  * `default_response_schema`- defines a default response schema for use in Swagger docs\n  * `default_callback_schema`- defines a default callback schema for use in Swagger docs (Callbacks)","ref":"Valspec.Controller.html#module-example"},{"type":"macro","title":"Valspec.Controller.valspec_create/3","doc":"Generates both a Goal validation struct and Swagger schema for a create controller action.\n\nExample:\n\n    valspec_create :create_client,\n    summary: \"Creates a client\",\n    response_schema: MyAppWeb.Response do\n      required(:name, :string, example: \"Client 1\")\n      required(:type, :enum, values: [:vendor, :customer])\n      required(:account_id, :uuid, example: \"82b557d1-c529-4aff-b0fb-55df16add1b5\")\n    end\n\n# Options:\n  * `summary`- a summary of what the post request does.\n  * `response_schema`- a response schema defined with the valspec_schema macro.\n  * `callback_schemas`- a list of tuples with the name of a callback to a schema. Example:\n    `\n    [{\"Create User Webhook\", MyApp.User.WebhookSchema}, {\"Create Account Webhook\", MyApp.User.WehookSchema}]\n    `","ref":"Valspec.Controller.html#valspec_create/3"},{"type":"macro","title":"Valspec.Controller.valspec_custom/2","doc":"Generates both a Goal validation struct and Swagger schema for a custom controller action.","ref":"Valspec.Controller.html#valspec_custom/2"},{"type":"macro","title":"Example 1: - Valspec.Controller.valspec_custom/2","doc":"valspec_custom(:my_action, summary: \"Hello world\")","ref":"Valspec.Controller.html#valspec_custom/2-example-1"},{"type":"macro","title":"Example 2: - Valspec.Controller.valspec_custom/2","doc":"valspec_custom\n        :my_validation_name\n        :my_action,\n        summary: \"Hello world\" do\n            required(:id, :string)\n            optional(:name, :string)\n            optional(:location_id, :string,\n                format: \"uuid\",\n                example: \"b388a4da-a3d6-42d4-8a98-4149f0007e6b\",\n                description: \"the location_id of something...\"\n            )\n      end","ref":"Valspec.Controller.html#valspec_custom/2-example-2"},{"type":"macro","title":"Options: - Valspec.Controller.valspec_custom/2","doc":"* `summary` - Swagger endpoint summary\n  * `response_schema` - The schema module to generating the response schema","ref":"Valspec.Controller.html#valspec_custom/2-options"},{"type":"macro","title":"Valspec.Controller.valspec_custom/4","doc":"","ref":"Valspec.Controller.html#valspec_custom/4"},{"type":"macro","title":"Valspec.Controller.valspec_delete/1","doc":"Generates both a Goal validation struct and Swagger schema for a delete controller action.","ref":"Valspec.Controller.html#valspec_delete/1"},{"type":"macro","title":"Example: - Valspec.Controller.valspec_delete/1","doc":"valspec_delete(\n        summary: \"Deletes a client\",\n        response_schema: MyAppWeb.Response\n      )","ref":"Valspec.Controller.html#valspec_delete/1-example"},{"type":"macro","title":"Options: - Valspec.Controller.valspec_delete/1","doc":"* `summary` - Swagger endpoint summary\n  * `response_schema` - The schema module to generating the response schema\n  * `callback_schemas`- a list of tuples with the name of a callback to a schema. Example:\n    `\n    [{\"Delete User Webhook\", MyApp.User.WebhookSchema}, {\"Delete Account Webhook\", MyApp.User.WehookSchema}]\n    `","ref":"Valspec.Controller.html#valspec_delete/1-options"},{"type":"macro","title":"Valspec.Controller.valspec_index/1","doc":"Generates both a Goal validation struct and Swagger schema for an index controller action.\n\nExample:\n\n    valspec_index(\n      summary: \"Lists all clients\",\n      response_schema:  MyAppWeb.IndexResponse,\n    )\n\n# Options:\n  * `summary`- a summary of what the post request does.\n  * `response_schema`- a response struct defined with the valspec_schema macro.","ref":"Valspec.Controller.html#valspec_index/1"},{"type":"macro","title":"Valspec.Controller.valspec_show/1","doc":"Generates both a Goal validation struct and Swagger schema for a show controller action.\n\nExample:\n\n    valspec_show(\n      summary: \"Show a single client\",\n      response_schema: MyAppApi.V3.Client.JSON.IndexResponse\n    )\n\n# Options:\n  * `summary`- a summary of what the post request does.\n  * `response_schema`- a response struct defined with the valspec_schema macro.","ref":"Valspec.Controller.html#valspec_show/1"},{"type":"macro","title":"Valspec.Controller.valspec_update/3","doc":"Generates both a Goal validation struct and Swagger schema for an update controller action.\n\nExample:\n\n    valspec_update :update_client,\n      summary: \"Updates a client\",\n      response_schema: MyAppWeb.Response do\n      optional(:client_id, :uuid, example: \"82b557d1-c529-4aff-b0fb-55df16add1b5\")\n\n      optional(:location_id, :string,\n        format: \"uuid\",\n        example: \"b388a4da-a3d6-42d4-8a98-4149f0007e6b\",\n        description: \"the location_id used to do stuff...\"\n      )\n    end\n\n# Options:\n  * `summary`- a summary of what the post request does.\n  * `response_schema`- a response schema defined with the valspec_schema macro.\n  * `callback_schemas`- a list of tuples with the name of a callback to a schema. Example:\n    `\n    [{\"Update User Webhook\", MyApp.User.WebhookSchema}, {\"Update Account Webhook\", MyApp.User.WehookSchema}]\n    `","ref":"Valspec.Controller.html#valspec_update/3"},{"type":"module","title":"Valspec.Schema","doc":"A module dedicated to generating Swagger schemas.\n\nSee valspec_schema/1 for more information.","ref":"Valspec.Schema.html"},{"type":"macro","title":"Valspec.Schema.valspec_schema/1","doc":"Generates both an embedded_schema and a Swagger schema.","ref":"Valspec.Schema.html#valspec_schema/1"},{"type":"macro","title":"Example: - Valspec.Schema.valspec_schema/1","doc":"valspec_schema do\n      field :id, :uuid, example: \"82b557d1-0000-0000-0000-55df16add1b5\"\n      field :name, :string, example: \"opts world\"\n      field :type, :enum, values: [:msp, :idn, :facility, :custom]\n      field :account_id, :uuid\n      field :location_id, :uuid\n      field :updated_at, :utc_datetime\n    end","ref":"Valspec.Schema.html#valspec_schema/1-example"},{"type":"module","title":"Valspec.Utils","doc":"","ref":"Valspec.Utils.html"},{"type":"function","title":"Valspec.Utils.add_callback_schema/1","doc":"","ref":"Valspec.Utils.html#add_callback_schema/1"},{"type":"function","title":"Valspec.Utils.generate_embedded_schema/1","doc":"Generates a Ecto.Schema.t() from a valspec definition.","ref":"Valspec.Utils.html#generate_embedded_schema/1"},{"type":"function","title":"Valspec.Utils.generate_swagger_schema/1","doc":"Generates a OpenApiSpex.Schema.t() from a valspec definition.","ref":"Valspec.Utils.html#generate_swagger_schema/1"},{"type":"function","title":"Valspec.Utils.maybe_add_callback_schemas/3","doc":"","ref":"Valspec.Utils.html#maybe_add_callback_schemas/3"},{"type":"function","title":"Valspec.Utils.maybe_add_response_schema/3","doc":"Helper to possibly add the response schema opt to the Swagger operation","ref":"Valspec.Utils.html#maybe_add_response_schema/3"},{"type":"extras","title":"Valspec","doc":"# Valspec\n\n**An elixir library for Phoenix that generates Swagger documentation and performs parameter validation.**","ref":"readme.html"},{"type":"extras","title":"Installation - Valspec","doc":"The latest version of Valspec is built on top [open_api_spex](https://github.com/open-api-spex/open_api_spex). You will need to install it along with the Valspec dependency:\n\n```elixir\ndef deps do\n  [\n    {:open_api_spex, \"~> 3.21\"},\n    {:valspec, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Adding to your project - Valspec","doc":"","ref":"readme.html#adding-to-your-project"},{"type":"extras","title":"Define your spec - Valspec","doc":"The first thing you will need to do is define your `open_api_spex` spec definition.\nVisit the [Generate Spec](https://github.com/open-api-spex/open_api_spex?tab=readme-ov-file#generate-spec) section in the github repo for more information.","ref":"readme.html#define-your-spec"},{"type":"extras","title":"Serving your spec (Running Swagger UI) - Valspec","doc":"Visit the [Serve Spec](https://github.com/open-api-spex/open_api_spex?tab=readme-ov-file#serve-the-spec) section in the github repo for more information on how to run the UI.","ref":"readme.html#serving-your-spec-running-swagger-ui"},{"type":"extras","title":"Add Valspec to your controller - Valspec","doc":"```elixir\n  defmodule MyAppWeb.ExampleUsersController do\n    use MyAppWeb, :controller\n    use Valspec.Controller\n\n    valspec_create :create_user, summary: \"Creates a User\" do\n      required(:first_name, :string, example: \"Greg\")\n      required(:role, :enum, values: [:admin, :normal], default: :normal)\n      optional(:last_name, :string, example: \"Jones\")\n      optional(:age, :integer, minimum: 18)\n    end\n\n    def create(conn, params) do\n      # `valspec_validate/2` - validates a map of parameters. Returns:\n      #    - {:ok, map()} when params are valid\n      #    - {:error, Ecto.%Changeset{}} when params are invalid\n      with {:ok, user_params} <- valspec_validate(:create_user, params) do\n        ...\n      else\n        {:error, %Ecto.Changeset{} = _changeset} ->\n          ...\n      end\n    end\n  end\n```","ref":"readme.html#add-valspec-to-your-controller"},{"type":"extras","title":"Creating a schema - Valspec","doc":"Using `valspex_schema/1` allows you to define response or callback schemas for your documentation. `valspex_schema/1` uses Ecto `embedded_schema` under the hood and uses its\nsyntax:\n\n```elixir\n  defmodule MyAppWeb.Users.Schema do\n    use Valspec.Schema\n\n    valspec_schema do\n      field :id, :uuid, example: \"82b557d1-0000-0000-0000-55df16add1b5\"\n      field :first_name, :string, example: \"Greg\"\n      field :last_name, :string, example: \"Jones\"\n      field :role, :enum, values: [:admin, :normal]\n      field :age, :integer\n    end\n  end\n\n  # You can use a valspec schema within other valspec schemas via `embeds_one/2` \n  # or `embeds_many/2`\n  defmodule MyApp.Users.SucessResponse do\n    use Valspec.Schema\n\n    valspec_schema do\n      embeds_one :data, MyAppWeb.Users.Schema\n    end\n  end\n```\n\nYou can use the schema by using the `default_response_schema` opt on init or by passing the `response_schema` to the valspec operation:\n\n```elixir\n  defmodule MyAppWeb.ExampleUsersController do\n    use MyAppWeb, :controller\n    use Valspec.Controller,\n    # Applies to all actions unless overridden\n    default_response_schema: MyApp.Users.SucessResponse\n    \n    # Uses MyApp.Users.SucessResponse\n    valspec_create :create_user, summary: \"Creates a User\" do\n      required(:first_name, :string, example: \"Greg\", nullable: true)\n    end\n\n    valspec_update \n      :update_user, \n      summary: \"Updates a User\"\n      # Overrides default MyApp.Users.SucessResponse\n      response_schema: MyApp.Users.UpdateSucessResponse do\n      required(:first_name, :string, example: \"Greg\")\n    end\n  end\n```","ref":"readme.html#creating-a-schema"},{"type":"extras","title":"Using schemas for type safety - Valspec","doc":"You can use your schemas directly in your responses to ensure your documentation is always up-to-date like so:\n\n```elixir\ndefmodule MyAppWeb.ExampleUsers.JSON do \n  def create(%{user: user}) do\n    %MyApp.Users.SucessResponse{data: data(user)}\n  end\n\n  defp data(%User{} = user) do\n    %MyAppWeb.Users.Schema{\n      id: user.id,\n      first_name: user.first_name,\n      last_name: user.last_name,\n      role: user.role,\n      age: user.age\n    }\n  end\nend\n```","ref":"readme.html#using-schemas-for-type-safety"},{"type":"extras","title":"List of operations - Valspec","doc":"The following macros are exposed:\n\n  * `valspec_create/3`- define specs for a controller's #create/2 action\n  * `valspec_update/3`- define specs for a controller's #update/2 action\n  * `valspec_index/1` - define specs for a controller's #index/2 action\n  * `valspec_delete/1` - define specs for a controller's #delete/2 action\n  * `valspec_show/1` - define specs for a controller's #show/2 action\n  * `valspec_custom/2` - define specs for a custom controller action.\n  * `valspec_custom/4` - define specs for a custom controller action.\n  * `valspec_schema/1`- defines a struct that can used in defining responses and callbacks.\n  * `valspec_validate/2` - validates a map of parameters.\n    Returns:\n      - `{:ok, map()}` when params are valid\n      - `{:error, Ecto.%Changeset{}}` when params are invalid","ref":"readme.html#list-of-operations"},{"type":"extras","title":"Examples - Valspec","doc":"There is an example phoenix project in this repo under `/examples/my_app`. You can run the app and navigate to the `/swaggerui#` to see the example","ref":"readme.html#examples"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.38.2"}}